package com.qmth.test.image;

import java.io.*;
import java.util.*;

import org.apache.commons.dbcp.BasicDataSource;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.support.rowset.SqlRowSet;

import com.qmth.test.image.model.Coordinate;
import com.qmth.test.image.model.Student;
import com.qmth.test.image.utils.CoverUtil;

public class SheetCover {

    public static Properties config = new Properties();

    private static JdbcTemplate template;

    public static List<Coordinate> coordinateList = new LinkedList<Coordinate>();

    public static Map<String, Student> studentMap = new HashMap<>();

    public static void main(String[] args) throws IOException {
         init(new FileInputStream(args[0]));
//        init(SheetTrack.class.getClassLoader().getResourceAsStream("cover.properties"));

        int examId = Integer.parseInt(config.getProperty("examId"));
        loadCoordinate(config.getProperty("cover.config"));
        loadStudent(examId, config.getProperty("student.file"));
        System.out.println("examId=" + examId + ", student count=" + studentMap.size());
        startCover(config.getProperty("sheet.path"), config.getProperty("output.path"));
    }

    private static void startCover(String sheetPath, String outPath) throws IOException {
        int count = 0;
        for (Student s : studentMap.values()) {
            coverStudent(s, sheetPath, outPath);
            count++;
        }
        System.out.println(count + " student cover finish");
    }

    public static void coverStudent(Student s, String sheetRoot, String sliceRoot) throws IOException {
        for (Coordinate c : coordinateList) {
            String examNumberEnd3 = s.examNumber.substring(s.examNumber.length() - 3, s.examNumber.length());
            File sheetFile = new File(new File(new File(sheetRoot, String.valueOf(s.examId)), examNumberEnd3),
                    s.examNumber + "-" + c.index + ".jpg");
            String secretNumberEnd3 = s.secretNumber.substring(s.secretNumber.length() - 3, s.secretNumber.length());
            File sliceFile = new File(new File(new File(sliceRoot, String.valueOf(s.examId)), secretNumberEnd3),
                    s.secretNumber + "-" + c.index + ".jpg");
            sliceFile.getParentFile().mkdirs();
            CoverUtil.coverImage(sheetFile, sliceFile, c.top, c.left, c.width, c.height);
            System.out.println(sheetFile.getAbsolutePath() + " -> " + sliceFile.getAbsolutePath());
        }
    }

    private static void loadStudent(int examId, String file) throws IOException {
        BufferedReader reader = new BufferedReader(new FileReader(file));
        String line = null;
        while ((line = reader.readLine()) != null) {
            SqlRowSet rs = template.queryForRowSet(
                    "select s.id,s.exam_number,s.sheet_count,s.secret_number "
                            + "from eb_exam_student s where s.exam_id=? "
                            + "and s.is_upload=1 and s.is_absent=0 and s.is_breach=0 " + "and s.exam_number=? ",
                    examId, line);
            while (rs.next()) {
                Student s = new Student();
                s.examId = examId;
                s.examNumber = rs.getString("exam_number");
                s.sheetCount = rs.getInt("sheet_count");
                s.secretNumber = rs.getString("secret_number");
                studentMap.put(s.examNumber, s);
            }
        }
        reader.close();
    }

    private static void loadCoordinate(String config) {
        coordinateList = Coordinate.build(config);
    }

    private static void init(InputStream ins) throws IOException {
        config.load(ins);

        BasicDataSource ds = new BasicDataSource();
        ds.setDriverClassName("com.mysql.cj.jdbc.Driver");
        ds.setUrl(config.getProperty("mysql.url"));
        ds.setUsername(config.getProperty("mysql.username"));
        ds.setPassword(config.getProperty("mysql.password"));

        template = new JdbcTemplate(ds);
    }
}
